@@ -27,9 +27,15 @@
 #include <gtk/gtkwindow.h>
 
 
-void       dlg_save_image   (GtkWindow    *parent,
-			     const char   *current_folder,
-			     GdkPixbuf    *pixbuf);
+typedef void (*ImageSavedFunc)       (char     *filename,
+				      gpointer  data);
+
+
+void       dlg_save_image   (GtkWindow      *parent,
+			     const char     *current_folder,
+			     GdkPixbuf      *pixbuf,
+			     ImageSavedFunc  done_func,
+			     gpointer        done_data);
 
 gboolean   dlg_save_options (GtkWindow    *parent,
 			     const char   *image_type,
