@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,11 +25,10 @@
 
 #include <libgnome/libgnome.h>
 #include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-master.h>
+#include <libgnomeprint/gnome-print-job.h>
 #include <libgnomeprintui/gnome-print-preview.h>
-#include <libgnomeprintui/gnome-print-master-preview.h>
+#include <libgnomeprintui/gnome-print-job-preview.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
-#include <libgnomeprintui/gnome-printer-dialog.h>
 #include <libgnomeprintui/gnome-print-paper-selector.h>
 #include <libgnomecanvas/libgnomecanvas.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -40,14 +39,21 @@
 #include <pango/pango-layout.h>
 
 #include "glib-utils.h"
+#include "gconf-utils.h"
+#include "file-utils.h"
 #include "image-viewer.h"
 #include "pixbuf-utils.h"
 #include "comments.h"
 #include "gthumb-canvas-text.h"
+#include "preferences.h"
 
 #define GLADE_PRINT_FILE "gthumb_print.glade"
 #define PARAGRAPH_SEPARATOR 0x2029	
-#define CANVAS_ZOOM 0.30 /*0.25*/
+#define CANVAS_ZOOM 0.25 /*0.30*/
+
+#define COMMENT_FONT_NAME "Sans Serif"
+#define COMMENT_FONT_SIZE 10
+
 
 typedef struct {
 	int                  ref_count;
@@ -68,13 +74,15 @@ typedef struct {
 	double               paper_bmargin;
 
 	GnomePrintConfig    *config;
-	GnomePrintMaster    *gpm;
+	GnomePrintJob       *gpj;
 
 	gboolean             print_comment;
 	gboolean             portrait;
 
 	/* Image Information. */
 
+	char                *image_path;
+
 	GdkPixbuf           *pixbuf;
 	
 	char                *comment;
@@ -117,9 +125,10 @@ print_info_unref (PrintInfo *pi)
 	pi->ref_count--;
 
 	if (pi->ref_count == 0) {
-		if (pi->gpm != NULL)
-			g_object_unref (pi->gpm);
+		if (pi->gpj != NULL)
+			g_object_unref (pi->gpj);
 		gnome_print_config_unref (pi->config);
+		g_free (pi->image_path);
 		g_object_unref (pi->pixbuf);
 		g_object_unref (pi->font_comment);
 		g_free (pi->comment);
@@ -298,6 +307,8 @@ print_comment (GnomePrintContext *pc,
 	int         next_paragraph_start;
 	char       *text_end;
 
+	gnome_print_setfont (pc, pi->font_comment);
+
 	p = pi->comment;
 	text_end = pi->comment + strlen (pi->comment);
 
@@ -325,6 +336,13 @@ print_comment (GnomePrintContext *pc,
 			y -= 1.2 * gnome_font_get_size (pi->font_comment);
 			
 			if ((y - pi->paper_bmargin) < fontheight) {
+				/* FIXME
+				gnome_print_showpage (pc);
+				gnome_print_beginpage (pc, NULL);
+
+				x = pi->paper_lmargin + MAX (0, (printable_width - width) / 2);
+				y = pi->paper_bmargin + height;
+				*/
 				/* text do not fit. */
 				/*return;*/
 			}
@@ -332,6 +350,13 @@ print_comment (GnomePrintContext *pc,
 			y = print_paragraph (pc, pi, p, end, x, y);
 			
 			if ((y - pi->paper_bmargin) < fontheight) {
+				/* FIXME
+				gnome_print_showpage (pc);
+				gnome_print_beginpage (pc, NULL);
+
+				x = pi->paper_lmargin + MAX (0, (printable_width - width) / 2);
+				y = pi->paper_bmargin + height;
+				*/
 				/* text do not fit. */
 				/*return;*/
 			}
@@ -364,7 +389,7 @@ print_image (GnomePrintContext *pc,
 	bmargin = pi->paper_bmargin;
 	tmargin = pi->paper_tmargin;
 
-	gnome_print_beginpage (pc, "1");
+	gnome_print_beginpage (pc, NULL);
 
 	if (border) {
 		gnome_print_gsave (pc);
@@ -395,6 +420,11 @@ print_image (GnomePrintContext *pc,
         gnome_print_grestore (pc);
 
 	if (pi->print_comment) {
+		/* FIXME
+		gnome_print_showpage (pc);
+		gnome_print_beginpage (pc, NULL);
+		*/
+
 		gnome_print_gsave (pc);
 		print_comment (pc, pi);
 		gnome_print_grestore (pc);
@@ -548,6 +578,10 @@ add_image_preview (PrintInfo *pi,
 	layout_width = (int) ((pi->paper_width - lmargin - rmargin) * CANVAS_ZOOM);
 
 	if (pi->print_comment && (pi->comment != NULL)) {
+		const char *font_name;
+
+		font_name = gnome_font_get_name (pi->font_comment);
+
 		pi->ci_comment = gnome_canvas_item_new (
 					group,
 					gthumb_canvas_text_get_type (),
@@ -556,8 +590,8 @@ add_image_preview (PrintInfo *pi,
 					"y",             pi->paper_height - bmargin,
 					"layout_width",  layout_width,
 					"wrap_mode",     PANGO_WRAP_CHAR,
-					"font",          "Sans 12",
-					"size_points",   CANVAS_ZOOM * 12,
+					"font",          font_name,
+					"size_points",   CANVAS_ZOOM * COMMENT_FONT_SIZE,
 					"anchor",        GTK_ANCHOR_S,
 					"justification", GTK_JUSTIFY_LEFT,
 					"clip_width",    pi->paper_width - lmargin - rmargin,
@@ -709,7 +743,23 @@ print_cb (GtkWidget *widget,
 {
 	PrintInfo         *pi = data->pi;
 	GnomePrintContext *gp_ctx;
-	gdouble            x, y;
+	double             x, y;
+	char              *title;
+	GtkWidget         *dialog;
+	int                response;
+	char              *value;
+
+	/* Save options */
+
+	value = gnome_print_config_get (pi->config, GNOME_PRINT_KEY_PAPER_SIZE);
+	eel_gconf_set_string (PREF_PRINT_PAPER_SIZE, value);
+	g_free (value);
+
+	value = gnome_print_config_get (pi->config, GNOME_PRINT_KEY_PAGE_ORIENTATION);
+	eel_gconf_set_string (PREF_PRINT_PAPER_ORIENTATION, value);
+	g_free (value);
+
+	/**/
 
 	g_object_get (G_OBJECT (pi->ci_image),
 		      "x", &x,
@@ -725,13 +775,38 @@ print_cb (GtkWidget *widget,
 	pi->ref_count++;
 	gtk_widget_destroy (data->dialog); 
 
-	/* Print !! */
+	/* Gnome Print Dialog */
 
-	pi->gpm = gnome_print_master_new_from_config (pi->config);
-	gp_ctx = gnome_print_master_get_context (pi->gpm);
+	pi->gpj = gnome_print_job_new (pi->config);
+
+	if (pi->image_path != NULL)
+		title = g_strdup_printf (_("Print %s"), 
+					 file_name_from_path (pi->image_path));
+	else
+		title = g_strdup (_("Print Image"));
+	dialog = gnome_print_dialog_new (pi->gpj, title, 0);
+        response = gtk_dialog_run (GTK_DIALOG (dialog));
+        gtk_widget_destroy (dialog);
+
+	gp_ctx = gnome_print_job_get_context (pi->gpj);
 	print_image (gp_ctx, pi, FALSE);
-	gnome_print_master_close (pi->gpm);
-	gnome_print_master_print (pi->gpm);	
+	gnome_print_job_close (pi->gpj);
+
+        switch (response) {
+        case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
+		gnome_print_job_print (pi->gpj);	
+                break;
+
+        case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
+		gtk_widget_show (gnome_print_job_preview_new (pi->gpj, title));
+                break;
+
+        default:
+		break;
+        }
+ 
+	g_free (title);
+
 	print_info_unref (pi);
 }
 
@@ -838,18 +913,6 @@ page_update (DialogData *data)
 }
 
 
-static void
-switch_page_cb (GtkWidget       *widget,
-		GtkNotebookPage *page,
-		guint            page_num,
-		gpointer         data)
-{
-	if (page_num != 0)
-		return;
-	page_update ((DialogData*) data);
-}
-
-
 static void
 print_comment_cb (GtkWidget  *widget,
 		  DialogData *data)
@@ -859,6 +922,94 @@ print_comment_cb (GtkWidget  *widget,
 }
 
 
+static void
+portrait_toggled_cb (GtkToggleButton  *widget,
+		     DialogData       *data)
+{
+	if (! gtk_toggle_button_get_active (widget))
+		return;
+	gnome_print_config_set (data->pi->config, GNOME_PRINT_KEY_PAGE_ORIENTATION, "R0");
+	page_update (data);
+}
+
+
+static void
+landscape_toggled_cb (GtkToggleButton  *widget,
+		      DialogData       *data)
+{
+	if (! gtk_toggle_button_get_active (widget))
+		return;
+	gnome_print_config_set (data->pi->config, GNOME_PRINT_KEY_PAGE_ORIENTATION, "R90");
+	page_update (data);
+}
+
+
+static void
+size_letter_toggled_cb (GtkToggleButton  *widget,
+			DialogData       *data)
+{
+	if (! gtk_toggle_button_get_active (widget))
+		return;
+	gnome_print_config_set (data->pi->config, GNOME_PRINT_KEY_PAPER_SIZE, "USLetter");
+	page_update (data);
+}
+
+
+static void
+size_legal_toggled_cb (GtkToggleButton  *widget,
+		       DialogData       *data)
+{
+	if (! gtk_toggle_button_get_active (widget))
+		return;
+	gnome_print_config_set (data->pi->config, GNOME_PRINT_KEY_PAPER_SIZE, "USLegal");
+	page_update (data);
+}
+
+
+static void
+size_executive_toggled_cb (GtkToggleButton  *widget,
+			   DialogData       *data)
+{
+	if (! gtk_toggle_button_get_active (widget))
+		return;
+	gnome_print_config_set (data->pi->config, GNOME_PRINT_KEY_PAPER_SIZE, "Executive");
+	page_update (data);
+}
+
+
+static void
+size_a3_toggled_cb (GtkToggleButton  *widget,
+		    DialogData       *data)
+{
+	if (! gtk_toggle_button_get_active (widget))
+		return;
+	gnome_print_config_set (data->pi->config, GNOME_PRINT_KEY_PAPER_SIZE, "A3");
+	page_update (data);
+}
+
+
+static void
+size_a4_toggled_cb (GtkToggleButton  *widget,
+		    DialogData       *data)
+{
+	if (! gtk_toggle_button_get_active (widget))
+		return;
+	gnome_print_config_set (data->pi->config, GNOME_PRINT_KEY_PAPER_SIZE, "A4");
+	page_update (data);
+}
+
+
+static void
+size_a5_toggled_cb (GtkToggleButton  *widget,
+		    DialogData       *data)
+{
+	if (! gtk_toggle_button_get_active (widget))
+		return;
+	gnome_print_config_set (data->pi->config, GNOME_PRINT_KEY_PAPER_SIZE, "A5");
+	page_update (data);
+}
+
+
 void
 print_image_dlg (GtkWindow   *parent, 
 		 ImageViewer *viewer,
@@ -866,9 +1017,11 @@ print_image_dlg (GtkWindow   *parent,
 {
 	PrintInfo    *pi;
 	DialogData   *data;
-	GtkWidget    *paper_vbox, *printer_vbox;
-	GtkWidget    *notebook;
 	CommentData  *cdata = NULL;
+	char         *title = NULL;
+	GtkWidget    *radio_button;
+	char         *value;
+	char         *button_name;
 
 	if (image_viewer_is_void (viewer))
 		return;
@@ -881,11 +1034,8 @@ print_image_dlg (GtkWindow   *parent,
 		GnomeVFSURI  *uri = gnome_vfs_uri_new (location);
 
 		if (gnome_vfs_uri_is_local (uri)) {
-			char *path;
-
-			path = gnome_vfs_get_local_path_from_uri (location);
-			cdata = comments_load_comment (path);
-			g_free (path);
+			pi->image_path = gnome_vfs_get_local_path_from_uri (location);
+			cdata = comments_load_comment (pi->image_path);
 		}
 
 		gnome_vfs_uri_unref (uri);
@@ -906,13 +1056,12 @@ print_image_dlg (GtkWindow   *parent,
 	}
 	g_object_ref (pi->pixbuf);
 
-	pi->image_w = (gdouble) gdk_pixbuf_get_width (pi->pixbuf);
-	pi->image_h = (gdouble) gdk_pixbuf_get_height (pi->pixbuf);
+	pi->image_w = (double) gdk_pixbuf_get_width (pi->pixbuf);
+	pi->image_h = (double) gdk_pixbuf_get_height (pi->pixbuf);
 	pi->portrait = TRUE;
 	pi->use_colors = TRUE;
 
-	/* FIXME */
-	pi->font_comment = gnome_font_find_closest_from_full_name ("Sans 12"); 
+	pi->font_comment = gnome_font_find_closest (COMMENT_FONT_NAME, COMMENT_FONT_SIZE);
 
 	data = g_new (DialogData, 1);
 	data->pi = pi;
@@ -929,27 +1078,52 @@ print_image_dlg (GtkWindow   *parent,
 	data->dialog = glade_xml_get_widget (data->gui, "page_setup_dialog");
 	data->print_comment_checkbutton = glade_xml_get_widget (data->gui, "print_comment_checkbutton");
 	data->btn_close = glade_xml_get_widget (data->gui, "btn_close");
-	data->hscale1 = glade_xml_get_widget (data->gui, "hscale1");
+	data->hscale1 = glade_xml_get_widget (data->gui, "hscale");
 	data->btn_center = glade_xml_get_widget (data->gui, "btn_center");
 	data->btn_print = glade_xml_get_widget (data->gui, "btn_print");
 
-	printer_vbox = glade_xml_get_widget (data->gui, "printer_vbox");
-	paper_vbox   = glade_xml_get_widget (data->gui, "paper_vbox");
-	notebook     = glade_xml_get_widget (data->gui, "notebook");
-
-	gtk_box_pack_start (GTK_BOX (printer_vbox),
-			    gnome_printer_selection_new (pi->config), 
-			    FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (paper_vbox),
-			    gnome_paper_selector_new (pi->config), /*FIXME*/
-			    FALSE, FALSE, 0);
-
 	pi->canvas = glade_xml_get_widget (data->gui, "canvas");
 
 	/* Set widgets data. */
 
 	gtk_widget_set_sensitive (data->print_comment_checkbutton, pi->comment != NULL);
 
+	value = eel_gconf_get_string (PREF_PRINT_PAPER_SIZE);
+	gnome_print_config_set (pi->config, GNOME_PRINT_KEY_PAPER_SIZE, value);
+
+	if (strcmp (value, "USLetter") == 0)
+		button_name = "print_size_letter_radiobutton";
+	else if (strcmp (value, "USLegal") == 0)
+		button_name = "print_size_legal_radiobutton";
+	else if (strcmp (value, "Executive") == 0)
+		button_name = "print_size_executive_radiobutton";
+	else if (strcmp (value, "A4") == 0)
+		button_name = "print_size_a4_radiobutton";
+	else if (strcmp (value, "A5") == 0)
+		button_name = "print_size_a5_radiobutton";
+	else if (strcmp (value, "A3") == 0)
+		button_name = "print_size_a3_radiobutton";
+
+	radio_button = glade_xml_get_widget (data->gui, button_name);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button), TRUE);
+	
+	g_free (value);
+
+	/**/
+
+	value = eel_gconf_get_string (PREF_PRINT_PAPER_ORIENTATION);
+	gnome_print_config_set (pi->config, GNOME_PRINT_KEY_PAGE_ORIENTATION, value);
+
+	if (strcmp (value, "R0") == 0)
+		button_name = "print_orient_portrait_radiobutton";
+	else if (strcmp (value, "R90") == 0)
+		button_name = "print_orient_landscape_radiobutton";
+
+	radio_button = glade_xml_get_widget (data->gui, button_name);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button), TRUE);
+
+	g_free (value);
+
 	/* Set the signals handlers. */
 
 	g_signal_connect (G_OBJECT (data->dialog), 
@@ -968,15 +1142,59 @@ print_image_dlg (GtkWindow   *parent,
 			  "clicked",
 			  G_CALLBACK (print_cb),
 			  data);
-	g_signal_connect (G_OBJECT (notebook), 
-			  "switch_page",
-			  G_CALLBACK (switch_page_cb),
-			  data);
 	g_signal_connect (G_OBJECT (data->print_comment_checkbutton),
 			  "toggled",
 			  G_CALLBACK (print_comment_cb),
 			  data);
 
+	radio_button = glade_xml_get_widget (data->gui, "print_orient_portrait_radiobutton");
+	g_signal_connect (G_OBJECT (radio_button),
+			  "toggled",
+			  G_CALLBACK (portrait_toggled_cb),
+			  data);
+
+	radio_button = glade_xml_get_widget (data->gui, "print_orient_landscape_radiobutton");
+	g_signal_connect (G_OBJECT (radio_button),
+			  "toggled",
+			  G_CALLBACK (landscape_toggled_cb),
+			  data);
+
+	radio_button = glade_xml_get_widget (data->gui, "print_size_letter_radiobutton");
+	g_signal_connect (G_OBJECT (radio_button),
+			  "toggled",
+			  G_CALLBACK (size_letter_toggled_cb),
+			  data);
+
+	radio_button = glade_xml_get_widget (data->gui, "print_size_legal_radiobutton");
+	g_signal_connect (G_OBJECT (radio_button),
+			  "toggled",
+			  G_CALLBACK (size_legal_toggled_cb),
+			  data);
+
+	radio_button = glade_xml_get_widget (data->gui, "print_size_executive_radiobutton");
+	g_signal_connect (G_OBJECT (radio_button),
+			  "toggled",
+			  G_CALLBACK (size_executive_toggled_cb),
+			  data);
+
+	radio_button = glade_xml_get_widget (data->gui, "print_size_a4_radiobutton");
+	g_signal_connect (G_OBJECT (radio_button),
+			  "toggled",
+			  G_CALLBACK (size_a4_toggled_cb),
+			  data);
+
+	radio_button = glade_xml_get_widget (data->gui, "print_size_a5_radiobutton");
+	g_signal_connect (G_OBJECT (radio_button),
+			  "toggled",
+			  G_CALLBACK (size_a5_toggled_cb),
+			  data);
+
+	radio_button = glade_xml_get_widget (data->gui, "print_size_a3_radiobutton");
+	g_signal_connect (G_OBJECT (radio_button),
+			  "toggled",
+			  G_CALLBACK (size_a3_toggled_cb),
+			  data);
+
 	data->adj = gtk_range_get_adjustment (GTK_RANGE (data->hscale1));
 	g_signal_connect (G_OBJECT (data->adj), 
 			  "value_changed",
@@ -985,11 +1203,17 @@ print_image_dlg (GtkWindow   *parent,
 
 	/**/
 
+	if (pi->image_path != NULL)
+		title = g_strdup_printf (_("Print %s"), 
+					 file_name_from_path (pi->image_path));
+	gtk_window_set_title (GTK_WINDOW (data->dialog), title); 
+	g_free (title);
+
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), parent);
 
 	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (pi->canvas), 
 					  CANVAS_ZOOM);
 	page_update (data);
 
-	gtk_widget_show_all (data->dialog);
+	gtk_widget_show (data->dialog);
 }
